//
// FPGA PACMAN video H-line resampler
//
// Version : beta1
//
// Copyright(c) 2002 Tatsuyuki Satoh , All rights reserved
//
// Important !
//
// This program is freeware for non-commercial use. 
// An author does no guarantee about this program.
// You can use this under your own risk. 
//
//

module resample(sclk,shs,srgb8,dclk,dhs,drgb8);
input sclk;             // input pixel clock
input shs;              // HSYNC input
input [7:0] srgb8;       // RGB   input

input dclk;             // output pixel clock
input dhs;              // HSYNC input
output [7:0] drgb8;      // RGB output

//
// H position
parameter H_POS  = 109*32;
parameter H_STEP = 110; // (1.00==256) ,6.144Mhz / 14.318MHz

//
// input timming
//
reg [8:0] shpos;    // input capture postion
always @(posedge sclk)
begin
	shpos <= shs ? 0 : shpos + 1;
end

//
//output timming
//
reg [16:0] dhpos;
always @(posedge dclk)
begin
	dhpos <= dhs ? H_POS : dhpos + H_STEP;
end

//
// RGB capture(portA) & output(portB)
//
RAMB4_S8_S8 double_scan_ram (
	.CLKA(sclk),.ADDRA(shpos),.DOA(),.DIA(srgb8),.WEA(1'b1),.ENA(1'b1),.RSTA(1'b0),
	.CLKB(dclk),.ADDRB(dhpos[16:8]),.DOB(drgb8),.DIB(8'h0),.WEB(1'b0),.ENB(1'b1),.RSTB(1'b0));

endmodule
