//
// FPGA PACMAN Z80 interface for Basaro's Z80TOP
//
// Version : beta2
//
// Copyright(c) 2002,2003 Tatsuyuki Satoh , All rights reserved
//
// Important !
//
// This program is freeware for non-commercial use. 
// An author does no guarantee about this program.
// You can use this under your own risk. 
//

//
// Additional circuit.
//
// 1.CPU clock synchronizer (when RESET_N raise timming)
// 2.WaitState Controller
//

//
// necessary lisence:
//
// see head comment in z80top.vhd or other source of Basaro's Z80IP.

// >>>>>>>>>> Extraction from z80top.vhd >>>>>>>>>>
//
// This program source is Intellectural Property(IP) by FPGA Informatoin in Japan.
// You could change this source code if you want. 
// If you want to use this program source (chanege or no change), you must buy 
// products that is we selected.
// If do you have questions, Please contact us. (basaro@fa2.so-net.ne.jp)
//
//                                      FPGA Infomation. 
//                                      Tadaaki Koyama.
// <<<<<<<<<< Extraction from z80top.vhd <<<<<<<<<<

// necessary files:
//
// 1.Z80TOP.NGC(Write protected) : synthesized Z80TOP
// 2.Z80TOP.jhd(Write protected) : Z80TOP tree file
// 2.Z80TOP.V                    : Z80TOP Verilog I/F file
//

module Z80IP(
	A,	DIN,	DOUT,	BUSWO,
	RESET_N,	INT_N,	NMI_N, WAIT_N,
	M1_N,	MREQ_N,	IORQ_N,	RD_N,	WR_N,
	RFSH_N,	HALT_N,	CLK2X,CLK);

// I/O assign
output [15:0] A;
input  [7:0] DIN;
output [7:0] DOUT;
input  RESET_N,INT_N,NMI_N,WAIT_N,CLK2X,CLK;
output M1_N,MREQ_N,IORQ_N,RD_N,WR_N,RFSH_N,HALT_N,BUSWO;

//
// CLK syncronizer
//
reg reset_sync = 1'b0;
always @(negedge CLK2X)
begin
  if(~CLK)
    reset_sync <= RESET_N;
end

//
// TW cycle detector
//
reg before_idle,TW_in,TW;
reg clk_disable;
wire access_cycle = (MREQ_N & IORQ_N);
always @(posedge CLK or negedge access_cycle)
begin
  if(~access_cycle)
  begin
    before_idle <= 1'b0;
    TW_in <= 1'b0;
  end else begin
    before_idle <= ~access_cycle;
    TW_in <= before_idle & access_cycle;
  end
end

//
// TW wait cycle generator
//
always @(negedge CLK or negedge access_cycle)
begin
  if(~access_cycle)
  begin
    TW <= 1'b0;
  end else begin
    if( TW_in | TW)
      TW <= ~WAIT_N;
  end
end

//
// skip CLK2X in WaitCycle
//
wire ZCLK = CLK2X | TW;

// Z80IP interface
Z80TOP z80core (
	.CLK(ZCLK),
	.RESET_N(reset_sync),
	.INT_N(INT_N),
	.NMI_N(NMI_N),
	.A(A),
	.D(DOUT),
	.DIN(DIN),
	.M1_N(M1_N),
	.MREQ_N(MREQ_N),
	.IORQ_N(IORQ_N),
	.RD_N(RD_N),
	.WR_N(WR_N),
	.BUSWO(BUSWO),
	.RFSH_N(RFSH_N),
	.HALT_N(HALT_N)
);

endmodule
